/* ------------------------------------------------------------------------------
    Library script for Milestones tab
    Author      : Sven Dickmans, Autodesk
    Imports     : -
    Last Update : Oct 22, 2020
------------------------------------------------------------------------------ */


/* ----------------------------------------------------------------------------------------------------
    Sync a milestone with a given date (will create, update or delete milestone)
    - name          : Name of the workflow step to which the miltestone should be attached to
    - event         : milestone event type (ENTER or EXIT)
    - date          : milestone taget date (e.g. item.TARGET_DATE)
    - threshold     : milestone threshold
    - progress      : progress to be set upon milestone completion
    - deleteIfEmpty : should the milestone be removed if target date is null (true | false)
   ---------------------------------------------------------------------------------------------------- */
function setMilestone(name, event, date, threshold, progress, deleteIfEmpty) {
    
    var milestoneIndex = -1;
    var targetDate     = new Date();
    
    if(         date !== null) targetDate    = date;
    if(        event === null) event         = "ENTER";
    if(    threshold === null) threshold     = 0;
    if(deleteIfEmpty === null) deleteIfEmpty = true;
    
    var day = targetDate.getDay();
    
         if(day === 0) targetDate.setDate(targetDate.getDate() + 1);
    else if(day === 6) targetDate.setDate(targetDate.getDate() + 2);

    for(var i = 0; i < item.milestones.length; i++) {
        if(item.milestones[i].workflowStateName == name) {
            if(item.milestones[i].milestoneType == event) {
                milestoneIndex = i;
                break;
            }
        }
    }
    
    if(deleteIfEmpty) {
        if (date === null) {
            if(milestoneIndex > -1) {
                item.milestones.splice(milestoneIndex, 1);
            }
            return;
        }
    }

    if(milestoneIndex == -1) {
        
		var milestone = item.addMilestone();
		    milestone.setWorkflowState(name);
		    milestone.milestoneType = event;
		    milestone.milestoneDate = targetDate;
		    milestone.warnThreshold = 0;
            milestone.progress = progress;
        
    } else {
        
        item.milestones[milestoneIndex].milestoneDate = targetDate;
        item.milestones[milestoneIndex].progress = progress;
        
    }
    
}


/* ----------------------------------------------------------------------------------------------------
    Create a milestone based on offset days
    Ensures not to schedule milestones during weekends
    - name     : name of the workflow state for which the milestone should be created
    - event    : milestone event type (ENTER or EXIT)
    - offset   : offset days from today to calculate target date
    - progress : milestone progress to set
   ---------------------------------------------------------------------------------------------------- */
function setStandardMilestone(name, event, offset, progress) {
    
    var targetDate = new Date();
        targetDate.setDate(targetDate.getDate() + offset);

    var day = targetDate.getDay();
    
         if(day === 0) targetDate.setDate(targetDate.getDate() + 1);
    else if(day === 6) targetDate.setDate(targetDate.getDate() + 2);

	var milestone = item.addMilestone();
	    milestone.setWorkflowState(name);
	    milestone.milestoneType = event;
	    milestone.milestoneDate = targetDate;
	    milestone.warnThreshold = 0;
        milestone.progress = progress;
    
}


/* ----------------------------------------------------------------------------------------------------
    Update progres of a defined milestone
        - name          : Name of the workflow step to which the miltestone should be attached to
        - progress      : progress to be set upon milestone completion
   ---------------------------------------------------------------------------------------------------- */
function setMilestoneProgress(name, progress) {
    
    for(var i = 0; i < item.milestones.length; i++) {
        
        var stateName = item.milestones[i].workflowStateName;
        
        if(stateName === "" || stateName === name) {
            item.milestones[i].progress = progress;
        }
        
    }
    
}


/* ----------------------------------------------------------------------------------------------------
    Retrieve target date of a defined workflow step
   ---------------------------------------------------------------------------------------------------- */
function getMilestoneTargetDate(workflowStateName) {
    
    var milestones = item.milestones;
    
    if(milestones !== null) {
        
        for(var i = 0; i < milestones.length; i++) {

            if(workflowStateName === milestones[i].workflowStateName) {
                return milestones[i].milestoneDate;
            }
            
        }
        
    }
    
    return null;
    
}


/* ----------------------------------------------------------------------------------------------------
    Remove given existing milestones
   ---------------------------------------------------------------------------------------------------- */
function removeMilestones(name) {
    
    for(var i = item.milestones.length - 1; i >= 0; i--) {
        if(item.milestones[i].workflowStateName === name) {
            item.milestones.splice(i, 1);
        }
    }
}


/* ----------------------------------------------------------------------------------------------------
    Remove all existing milestones
   ---------------------------------------------------------------------------------------------------- */
function clearMilestones() {
    
    for(var i = 0; i < item.milestones.length; i++) {
        item.milestones.splice(i, 1);
    } 
    
}



/* ----------------------------------------------------------------------------------------------------
    Resets an existing milestone identified by its name
   ---------------------------------------------------------------------------------------------------- */
function resetMilestone(workflowStateName) {
    
    for(var i = 0; i < item.milestones.length; i++) {
        
        var milestone = item.milestones[i];
        
        if(milestone.workflowStateName === workflowStateName) {
            
            if(milestone.status === "Done") {
                
                var milestoneNew = item.addMilestone();
	                milestoneNew.setWorkflowState(workflowStateName);
	                milestoneNew.milestoneType = milestone.milestoneType;
	                milestoneNew.milestoneDate = milestone.milestoneDate;
	                milestoneNew.warnThreshold = milestone.warnThreshold;
                    milestoneNew.progress      = milestone.progress;
                
                item.milestones.splice(i, 1);
                
            }
            
            return;
            
        }
        
    }
    
}