/* ----------------------------------------------------------------------------------------------------
    General library script
    Author : Sven Dickmans, Autodesk
    Imports : -
    Last Update : May 7, 2019
   ---------------------------------------------------------------------------------------------------- */


/* ----------------------------------------------------------------------------------------------------
    Clone current item into (another workspace)
    - source    : Item to clone
    - workspace : Internal name of workspace in which new item should be created
    ---------------------------------------------------------------------------------------------------- */
function cloneItem(source, workspace) {
    
    var newItem      = createItem(workspace);
    var fieldsSource = Object.keys(source);
    var fieldsTarget = Object.keys(newItem);
    
    for(var i = 0; i < fieldsSource.length; i++) {
        
        var fieldSource = fieldsSource[i];
        for(var j = 0; j < fieldsTarget.length; j++) {
            var fieldTarget = fieldsTarget[j];
            if(fieldSource == fieldTarget)  newItem[fieldTarget] = source[fieldSource];
        }
    
    }
  
    return newItem;
    
}


/* ----------------------------------------------------------------------------------------------------
    Add access to selected supplier for given item by adding supplier specific user groups as owner
    - elem           : item for which access should be granted
    - supplier       : supplier record who sould get access
    - fieldUserGroup : field of supplier details to get the given supplier's user group (e.g. USER_GROUP)
    ---------------------------------------------------------------------------------------------------- */
function addSupplierAccess(elem, supplier, fieldUserGroup) {

	if(supplier === null) return;	
        
    var userGroup = supplier[fieldUserGroup];
    
    if(userGroup === null) return;
    if(userGroup === "") return;

    var group = Security.loadGroup(userGroup);
    
    if(group === null) return;
    
    var additionalGroups = elem.master.groupAdditionalOwners;
		
    for(var i = 0; i < additionalGroups.length; i++) {
        if(additionalGroups[i] == userGroup) {
            return;
        }
    }
    
    additionalGroups.push(group);
    	    
}


/* ----------------------------------------------------------------------------------------------------
    Get user account based on user name (e.g. to set additional owners)
    - name : user's name (e.g. 'Dickmans, Sven')
    ---------------------------------------------------------------------------------------------------- */
function getUser(name) {
    
    var nameArray   = name.split(", ");
    var lName       = nameArray[0]; 
    var fName       = nameArray[1];
    var userINFO    = Security.searchUsers({lastName:lName, firstName:fName}); 
    
    return Security.loadUser(userINFO[0].id); 
    
}


/* ----------------------------------------------------------------------------------------------------
    Get user account for current user (replaces original getUserName script)
    ---------------------------------------------------------------------------------------------------- */
function getUserName() {
    var usr = Security.loadUser(userID);
    return usr.lastName + ", " + usr.firstName;
}


/* ----------------------------------------------------------------------------------------------------
    Sync additional owners with a pick list linked to workspaces Persons or a defined list of users
    - fieldSelection : ID of field which relates to Persons (whose selection will drive the owners) (optional)
    - userList       : array of user objects (optional)
    ---------------------------------------------------------------------------------------------------- */
function syncOwners(fieldSelection, userList) {
    
    
    if(fieldSelection !== null) { userList = getSelectedUsers(fieldSelection); }
    else if(userList === null) userList = [];
    
    for(var i = 0; i < userList.length; i++) {

        var add = true;
        var userItem = userList[i];
        
        for(var j = 0; j < item.master.additionalOwners.length; j++) {
            var owner = item.master.additionalOwners[j];
            if(owner === userItem) {
                add = false;
                break;
            }
        }

        if(add) item.master.additionalOwners.push(userItem);

    }
    
    if(userList.length !== item.master.additionalOwners.length) cleanupOwners(userList);
            
}
function getSelectedUsers(fieldSelection) {
    
    var selectedUsers = [];
    
    for(var i = 0; i < item[fieldSelection].length; i++) {

        var lastName = item[fieldSelection][i].LAST_NAME;

        if(lastName !== null) {

            var firstName = item[fieldSelection][i].FIRST_NAME;

            if(firstName !== null) {

            var userINFO  = Security.searchUsers({lastName:lastName, firstName:firstName}); 

                if(userINFO !== null) {
                    if(userINFO.length > 0) {

                        var userItem  = Security.loadUser(userINFO[0].id); 
                            
                        if(userItem !== null) {
                            selectedUsers.push(userItem);
                        }
                    }
                }
            }
        }
    }
    
    
    return selectedUsers;
    
}
function cleanupOwners(userList) {
    
    for(var i = item.master.additionalOwners.length - 1; i >= 0; i--) {
            
        var owner  = item.master.additionalOwners[i];
        var remove = true;
        
        for(var j = 0; j < userList.length; j++) {
    
            var userItem = userList[j];
            if(owner === userItem) {
                remove = false;
                continue;
            }
            
        }
            
        if(remove) item.master.additionalOwners.splice(i,1);
    }
    
}


/* ----------------------------------------------------------------------------------------------------
    Create follow up process, link each other and copy attributes
    - workspace          : ID of workspace in which new process needs to be created
    - linkBack           : ID of field in new workspaces to link back to current item
    - linkNext           : ID of field in current workspaces to link to new item
    - copyMatchingFields : toggle to copy values of fields having same field id (true/false)
    - copyManagedItems   : toggle to copy list of managed items to new object (true/false)
    ---------------------------------------------------------------------------------------------------- */
function createFollowUpProcess(workspace, linkBack, linkNext, copyMatchingFields, copyManagedItems) {
    
    var followUp = createItem(workspace);
    
    if(linkBack !== null) followUp[linkBack] = item;
    if(linkNext !== null) item[linkNext] = followUp;
    
    if(copyMatchingFields !== null) {
        if(copyMatchingFields) {
            
            var keys = Object.keys(item);
            
            for(var i = 0; i < keys.length; i++) {
                
                var key = keys[i];
                followUp[key] = item[key];
                
            }
            
        }
    }
    
    if(copyManagedItems !== null) {
        if(copyManagedItems) {
            for(var j = 0; j < item.workflowItems.length; j++) {
                var workflowItem = item.workflowItems[j].item;
                followUp.workflowItems.addItem({RELATED:workflowItem});
            }
        }
    }
    
    return followUp;
    
}



/* ----------------------------------------------------------------------------------------------------
    Capture workflow comment in paragraph field to save feedback in thread
    - field : Paragraph field to capture discussion thread
    - append : Append transition details (true) or save last transistion details only (false)
    ---------------------------------------------------------------------------------------------------- */
function appendToThread(field, append) {
    
    if(item.workflowActions[0].transition.fromState === null) return;
    if(item.workflowActions[0].comments === null) return;
    if(item.workflowActions[0].comments === "") return;
    
    var comment = item.workflowActions[0].comments;
    var date    = new Date();
    var thread  = "<p>";
    
    thread += "<span style='font-size:0.8em;font-style:italic;color:#ffa600;white-space:nowrap;'>";
    thread += item.workflowActions[0].transition.fromState.shortName + " &gt; " + item.workflowActions[0].transition.toState.shortName;
    thread += "</span>";
    thread += "<br>";
   
    if(comment !== null) {
        if(comment !== "") {
            thread += comment + "<br>";
        }
    }
    
    thread += "<span style='font-size:0.8em;font-style:italic;color:#999;white-space:nowrap;'>";
    thread += getUserName() + " (" + date.toDateString() + ")";
    thread += "</span>";
    thread += "</p>";
            
    if(append) {
        if(item[field] !== null) thread += item[field];
    }
            
    item[field] = thread;
    
}


// Save transition completion & comment in defined attributes
function saveProgress(fieldComment, fieldDate) {
    
    item[fieldComment] = item.workflowActions[0].comments;
    item[fieldDate] = new Date();
    
}


// Capture current date and compare it with target date
function saveCurrentAndDifference(propertyCurrent, propertyTarget, propertyDifference, enforce) {
    
    var current = new Date();
    var target  = item[propertyTarget];
    
    item[propertyCurrent] = current;
    
    if(target !== null) {
        
        if(item[propertyDifference] === null || enforce) {
            
            var difference = target - current;
                difference = difference / 1000 / 60 / 60 / 24;
            item[propertyDifference] = difference;
            
        }
        
    }
    
}


// Send Advanced Print View by Mail
function sendAPVByMail(printView, address, subject, sendCC) {
    
    if(sendCC === null) sendCC = false;
    
    var user = Security.loadUser(userID);
    var mail = user.email;
    
    var emailBody = getPrintView(printView);
        emailBody = emailBody.replace(/dmsid=XYZ/g, "dmsID=" + item.dmsId);
        emailBody = emailBody.replace(/mailto:XYZ/g, "mailto:" + mail);
        emailBody = emailBody.replace(/@XYZ/g, mail);
    
    var email = new Email();
        email.to = address;
        email.subject = subject;
        email.body = emailBody;
    
    if(sendCC) email.cc = mail;    
        
    email.send();
        
}


// set additional owners based on defined attribute
function setAdditionalOwner(fieldName) {
    
    var user     = Security.loadUser(item.master.owner);
    var userId   = "";    
    var username = item[fieldName];
    var current  = user.lastName + ", " + user.firstName;
    var owners   = item.master.additionalOwners;
    
    if(username === null) {
        
        username = "";
        
    } else {
        
        var nameArray = username.split(", ");
        var lName     = nameArray[0]; 
        var fName     = nameArray[1];
        var userIds   = Security.searchUsers({lastName:lName, firstName:fName});
            userId    = userIds[0];
        
    }
    
    var bAdd = current !== username;
    
    for(var i = owners.length - 1; i >= 0; i--) {
        if(userId === owners[i]) { bAdd = false; }
        else { owners.splice(i, 1); }
    }
    
    if(bAdd) {
        var userItem  = Security.loadUser(userId); 
        owners.push(userItem);
    }
    
}